/* Copyright (C) 1999 Southern Gold Development <tangomanrulz@geocities.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "gview.h"

/* No changes to code, only to format to look like mine does 
 * The readXVThumb function source may be re-used under
 * the XFree86-style license. <adam@gimp.org>
 */
   
guchar* readXVThumb(const gchar *fnam, gint* w, gint* h, gchar** imginfo) {
	FILE *fp;
	const gchar *P7_332 = "P7 332";
	gchar P7_buf[7];
	gchar linebuf[200];
	guchar *buf;
	gint twofivefive;
	void *ptr;

	*w = *h = 0;
	*imginfo = NULL;
	fp = fopen (fnam, "rb");
	if (!fp)
		return (NULL);
	fread (P7_buf, 6, 1, fp);
	if (strncmp(P7_buf, P7_332, 6)!=0) {
		g_warning("Thumbnail doesn't have the 'P7 332' header.");
		fclose(fp);
		return(NULL);
	}
	fread (P7_buf, 1, 1, fp);
	do {
		ptr = fgets(linebuf, 199, fp);
		if ((strncmp(linebuf, "#IMGINFO:", 9) == 0) && (linebuf[9] != '\0') && (linebuf[9] != '\n')) {
			if (linebuf[strlen(linebuf)-1] == '\n')
				linebuf[strlen(linebuf)-1] = '\0';

			if (linebuf[9] != '\0') {
				if (*imginfo)
					g_free(*imginfo);
					*imginfo = g_strdup (&linebuf[9]);
			}
		}
	} while (ptr && linebuf[0]=='#'); /* keep throwing away comment lines */
	if (!ptr) {
		fclose(fp);
		return(NULL);
	}
	sscanf(linebuf, "%d %d %d\n", w, h, &twofivefive);
	if (twofivefive!=255) {
		g_warning("Thumbnail is of funky depth.");
		fclose(fp);
		return(NULL);
	}
	if ((*w)<1 || (*h)<1 || (*w)>80 || (*h)>60) {
		g_warning ("Thumbnail size bad.  Corrupted?");
		fclose(fp);
		return (NULL);
	}
	buf = g_malloc((*w)*(*h));
	fread(buf, (*w)*(*h), 1, fp);
	fclose(fp);
	return(buf);
}

void handleFile(sgview *view, gchar *file) {
	gchar *wdir;
	int cnt;

	if (!(file)) 
		return;
	if (strcmp(file, _(".. (parent)")) == 0) {
		wdir = view->currPath;
		if (strlen(wdir) == 1) 
			return;
		cnt = strlen(wdir);
		wdir[cnt - 1] = '\0';
		cnt--;
		while ((cnt > 0) && (wdir[cnt] != '/')) {
			wdir[cnt] = '\0';
			cnt--;
		}
		view->currPath = g_strdup(wdir);
		g_free(wdir);
		gloaddir(view);
		return;
	}
	if (file[strlen(file) - 1] == '/') {
		wdir = g_strconcat(view->currPath, file, NULL);
		g_free(view->currPath);
		view->currPath = wdir;
		gloaddir(view);
		return;
	}
	if (file[0] == '/') {
		loadPic(view, file);
		return;
	}
	wdir = g_strconcat(view->currPath, file, NULL);
	loadPic(view, wdir);
	g_free(wdir);
}

#ifdef HAVE_WORKING_MSG
gint ipcIncoming(gpointer data) {
	nmsg ipc_msg;
	GtkAdjustment *adj;
	gint res, errIn, cnt, row, winB, rowH;
	gchar *spath, *iName, *work, *tmpN, *tmpF;
	gfloat newP;
	sgview *view;
	struct stat buff;
	
	view = (sgview *) data;
	gtk_timeout_remove(view->msgTimer); 
	if (view->dqueue != -1) {
		res = msgrcv(view->dqueue, (struct msgbuf *) &ipc_msg, MAX_BUFF, 4351, IPC_NOWAIT);
		if (res != -1) {
			work = g_strdup(ipc_msg.mtext);
			errIn = stat(work, &buff);
			if (errIn != -1) {
				if (buff.st_mode & S_IFDIR) {
					if (work[strlen(work) - 1] == '/')
						spath = g_strdup(work);
					else 
						spath = g_strconcat(work, "/", NULL);
					g_free(view->currPath);
					view->currPath = spath;
					gloaddir(view);
				}
				else {
					tmpN = g_strdup(work);
					tmpF = rindex(tmpN, '/');
					tmpF[0] = '\0';
					tmpF++;
					spath = g_strconcat(tmpN, "/", NULL);
					iName = g_strdup(tmpF);
					g_free (tmpN);
					g_free(view->currPath);
					view->currPath = spath;
					gloaddir(view);
					row = 0;
					for (cnt = 0; cnt < GTK_CLIST (view->clist)->rows; cnt++) {
						gtk_clist_get_text(GTK_CLIST (view->clist), cnt, 0, &tmpN);
						if (strcmp(tmpN, iName) == 0)
							row = cnt;
					}
					gtk_clist_unselect_row(GTK_CLIST (view->clist), 0, 0);
					GTK_CLIST (view->clist)->focus_row = row;
					gtk_clist_select_row(GTK_CLIST (view->clist), row, 0);
					adj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW (view->mscr));
					winB = adj->value + view->mscr->allocation.height;
					rowH = adj->upper / GTK_CLIST (view->clist)->rows;
					newP = rowH * row;
					if (newP > adj->upper - adj->page_size) 
						newP = adj->upper - adj->page_size;
					if (newP < adj->lower)
						newP = adj->lower;
					gtk_adjustment_set_value(adj, newP);
					handleFile(view, iName);
					g_free(iName);
				}
			}
			g_free(work);
		}
	}
	view->msgTimer = gtk_timeout_add(500, (GtkFunction) ipcIncoming, (gpointer) view);
	return 0;
}
#endif /* HAVE_WORKING_MSG */

void viewPic(GtkWidget *widget, gpointer data) {
	sgview *view;
	gchar *file;

	view = (sgview *) data;
	if (GTK_CLIST (view->clist)->focus_row == -1) 
		return;
	gtk_clist_get_text(GTK_CLIST (view->clist), GTK_CLIST(view->clist)->focus_row, 0, &file);
	handleFile(view, file);	
}

void quit(GtkWidget *widget, gpointer data) {
	sgview *view;

	view = (sgview *) data;
	gtk_widget_destroy(view->main);
}

void killwin(GtkWidget *widget, gpointer data) {
	sgview *view;

	view = (sgview *) data;
#ifdef HAVE_WORKING_MSG
	if (view->dqueue != -1) 
		msgctl(view->dqueue, IPC_RMID, NULL);
#endif /* HAVE_WORKING_MSG */
	g_free(view);
	gtk_main_quit();
}

void menuOpen(GtkWidget *widget, gpointer data) {
	sgview *view;
	gchar *file;

	view = (sgview *) data;
	if (GTK_CLIST (view->clist)->focus_row == -1)
		return;
	gtk_clist_get_text(GTK_CLIST (view->clist), GTK_CLIST(view->clist)->focus_row, 0, &file);

	handleFile(view, file);	
}

void menuInfo(GtkWidget *widget, gpointer data) {
	sgview *view;
	gchar *file;

	view = (sgview *) data;
	if (GTK_CLIST (view->clist)->focus_row == -1)
		return;
	gtk_clist_get_text(GTK_CLIST (view->clist), GTK_CLIST(view->clist)->focus_row, 0, &file);
	
}

void menuSet(GtkWidget *widget, gpointer data) {
	sgview *view;
	gchar *file, *lfile;
	GdkImlibImage *newImage;

	view = (sgview *) data;
	if (GTK_CLIST (view->clist)->focus_row == -1)
		return;
	gtk_clist_get_text(GTK_CLIST (view->clist), GTK_CLIST(view->clist)->focus_row, 0, &file);
	if (index(file, '/') != NULL)
		return;
	if (strcmp(file, _(".. (parent)")) == 0) 
		return;
	lfile = g_strconcat(view->currPath, file, NULL);
	newImage = gdk_imlib_load_image(lfile);
	g_free(lfile);
	view->backPic = newImage;
	setBack(widget, (gpointer) data);
}

void menuRefresh(GtkWidget *widget, gpointer data) {
	gloaddir((sgview *) data);
}

void menuHome(GtkWidget *widget, gpointer data) {
	sgview *view;
	
	view = (sgview *) data;
	g_free(view->currPath);
	view->currPath = g_strconcat(getenv("HOME"), "/", NULL);
	gloaddir(view);
}

/* Here i've made changes to fit with the rest of gView's structs and so on
 * Apologies to all those who wrote this code
 */

void doImagePreview (GtkWidget *widget, gint row, gint col, GdkEvent *event, gpointer data) {
	sgview *view;
	guchar *thumb_rgb, *raw_thumb;
	gint tnw,tnh;
	gchar *tname, *imginfo, *fname;

	view = (sgview *) data;
	gtk_clist_get_text(GTK_CLIST (view->clist), row, 0, &fname);
	tname = g_strconcat (view->currPath, G_DIR_SEPARATOR_S, ".xvpics", G_DIR_SEPARATOR_S, fname, NULL);
	raw_thumb = readXVThumb(tname, &tnw, &tnh, &imginfo);
	g_free (tname);

	if (raw_thumb) {
		int i;
		gtk_widget_hide (GTK_WIDGET (view->preview));
		thumb_rgb = g_malloc (3 * tnw * tnh);
		for (i=0; i<tnw*tnh; i++) {
			thumb_rgb[i*3  ] = ((raw_thumb[i]>>5)*255)/7;
			thumb_rgb[i*3+1] = (((raw_thumb[i]>>2)&7)*255)/7;
			thumb_rgb[i*3+2] = (((raw_thumb[i])&3)*255)/3;
		}
		gtk_preview_size (GTK_PREVIEW (view->preview), tnw, tnh);
		for (i=0; i<tnh; i++)
			gtk_preview_draw_row(GTK_PREVIEW (view->preview), &thumb_rgb[3*i*tnw], 0, i, tnw);
		gtk_widget_show (GTK_WIDGET (view->preview));
		gtk_widget_queue_draw (GTK_WIDGET (view->preview));
		g_free (thumb_rgb);
		g_free (raw_thumb);
	}
	else {
		if (imginfo)
			g_free(imginfo);
		gtk_widget_hide (GTK_WIDGET(view->preview));
    }
}


void popupMenu(sgview *view, GdkEventButton *event) {
	GtkWidget *menu, *item, *submenu;

	menu = gtk_menu_new();
	submenu = gtk_menu_new();


	item = gtk_menu_item_new_with_label(_("Add To Slideshow"));
	gtk_signal_connect(GTK_OBJECT (item), "activate", (GtkSignalFunc) enqueueSlide, (gpointer) view);
	gtk_menu_append(GTK_MENU (submenu), item);
	gtk_widget_show(item);

	item = gtk_menu_item_new_with_label(_("Start"));
	gtk_signal_connect(GTK_OBJECT (item), "activate", (GtkSignalFunc) startSlide, (gpointer) view);
	gtk_menu_append(GTK_MENU (submenu), item);
	gtk_widget_show(item);

	item = gtk_menu_item_new_with_label(_("Stop"));
	gtk_signal_connect(GTK_OBJECT (item), "activate", (GtkSignalFunc) stopSlide, (gpointer) view);
	gtk_menu_append(GTK_MENU (submenu), item);
	gtk_widget_show(item);

	item = gtk_menu_item_new_with_label(_("Pause"));
	gtk_signal_connect(GTK_OBJECT (item), "activate", (GtkSignalFunc) pauseSlide, (gpointer) view);
	gtk_menu_append(GTK_MENU (submenu), item);
	gtk_widget_show(item);

	item = gtk_menu_item_new_with_label(_("Resume"));
	gtk_signal_connect(GTK_OBJECT (item), "activate", (GtkSignalFunc) resumeSlide, (gpointer) view);
	gtk_menu_append(GTK_MENU (submenu), item);
	gtk_widget_show(item);

	item = gtk_menu_item_new_with_label(_("View Slide List"));
	gtk_signal_connect(GTK_OBJECT (item), "activate", (GtkSignalFunc) viewSlide, (gpointer) view);
	gtk_menu_append(GTK_MENU (submenu), item);
	gtk_widget_show(item);

	item = gtk_menu_item_new_with_label(_("Clear Slideshow"));
	gtk_signal_connect(GTK_OBJECT (item), "activate", (GtkSignalFunc) clearSlide, (gpointer) view);
	gtk_menu_append(GTK_MENU (submenu), item);
	gtk_widget_show(item);

	item = gtk_menu_item_new_with_label(_("Open"));
	gtk_signal_connect(GTK_OBJECT (item), "activate", (GtkSignalFunc) menuOpen, (gpointer) view);
	gtk_menu_append(GTK_MENU (menu), item);
	gtk_widget_show(item);

#ifdef HAVE_CURL
	item = gtk_menu_item_new_with_label(_("Open URL"));
	gtk_signal_connect(GTK_OBJECT (item), "activate", (GtkSignalFunc) menuOpenURL, (gpointer) view);
	gtk_menu_append(GTK_MENU (menu), item);
	gtk_widget_show(item);
#endif /* HAVE_CURL */

	item = gtk_menu_item_new_with_label(_("SlideShow"));
	gtk_menu_append(GTK_MENU (menu), item);
	gtk_menu_item_set_submenu(GTK_MENU_ITEM (item), submenu);
	gtk_widget_show(item);

	item = gtk_menu_item_new_with_label(_("Set As Wallpaper..."));
	gtk_signal_connect(GTK_OBJECT (item), "activate", (GtkSignalFunc) menuSet, (gpointer) view);
	gtk_menu_append(GTK_MENU (menu), item);
	gtk_widget_show(item);

	item = gtk_menu_item_new_with_label(_("Info"));
	gtk_signal_connect(GTK_OBJECT (item), "activate", (GtkSignalFunc) menuInfo, (gpointer) view);
	gtk_menu_append(GTK_MENU (menu), item);
	gtk_widget_show(item);

	item = gtk_menu_item_new_with_label(_("Refresh"));
	gtk_signal_connect(GTK_OBJECT (item), "activate", (GtkSignalFunc) menuRefresh, (gpointer) view);
	gtk_menu_append(GTK_MENU (menu), item);
	gtk_widget_show(item);

	item = gtk_menu_item_new_with_label(_("Delete"));
	gtk_signal_connect(GTK_OBJECT (item), "activate", (GtkSignalFunc) menuDelete, (gpointer) view);
	gtk_menu_append(GTK_MENU (menu), item);
	gtk_widget_show(item);

	item = gtk_menu_item_new_with_label(_("Rename"));
	gtk_signal_connect(GTK_OBJECT (item), "activate", (GtkSignalFunc) menuRename, (gpointer) view);
	gtk_menu_append(GTK_MENU (menu), item);
	gtk_widget_show(item);

	item = gtk_menu_item_new_with_label(_("Home"));
	gtk_signal_connect(GTK_OBJECT (item), "activate", (GtkSignalFunc) menuHome, (gpointer) view);
	gtk_menu_append(GTK_MENU (menu), item);
	gtk_widget_show(item);

	gtk_menu_popup(GTK_MENU (menu), NULL, NULL, NULL, NULL, event->button, event->time);
}

gint mouseClick(GtkWidget *widget, GdkEventButton *event, gpointer data) {
	sgview *view;
	gchar *file;

	view = (sgview *) data;
	if (GTK_CLIST (view->clist)->focus_row == -1)
		return FALSE;
	gtk_clist_get_text(GTK_CLIST (view->clist), GTK_CLIST(view->clist)->focus_row, 0, &file);
	switch (event->button) {
		case 1:
			if (event->type == GDK_2BUTTON_PRESS) 
				handleFile(view, file);
			break;
		default:
			if (event->type == GDK_BUTTON_PRESS) 
				popupMenu(view, event);
			break;
	} 
	return FALSE;
}

gint keyClick(GtkWidget *widget, GdkEventKey *event, gpointer data) {
	sgview *view;
	gchar *file;

	view = (sgview *) data;
	if (GTK_CLIST (view->clist)->focus_row == -1)
		return FALSE;
 	gtk_clist_get_text(GTK_CLIST (view->clist), GTK_CLIST(view->clist)->focus_row, 0, &file);

	switch (event->keyval) {
		case GDK_Return:
		case GDK_KP_Enter:
			handleFile(view, file);
			break;
		case GDK_X:
		case GDK_x:
			gtk_widget_destroy(view->main);
			break;
		case GDK_Delete:
		case GDK_KP_Delete:
			menuDelete(widget, data);
			break;
		case GDK_H:
		case GDK_h:
			menuHome(widget, data);
			break;
		default:
			break;
	}
	return FALSE;
}

void createWin(sgview *view) {
	GtkWidget *win, *hbox, *vbox, *btn, *clist, *scr, *lbl, *fra, *pix;
	gchar *titles[] = {_("File"), _("Time Stamp"), _("Size"), _("Type")};


	view->tips = gtk_tooltips_new();
	gtk_tooltips_enable(view->tips);

	win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	view->main = win;
	gtk_window_set_title(GTK_WINDOW (win), _("gView image viewer"));
	gtk_window_set_policy(GTK_WINDOW (win), TRUE, TRUE, TRUE);
	gtk_window_position(GTK_WINDOW (win), GTK_WIN_POS_NONE);
	gtk_signal_connect(GTK_OBJECT (win), "destroy", (GtkSignalFunc) killwin, (gpointer) view );
	gtk_widget_set_usize(GTK_WIDGET (win), 400, 475);
	gtk_container_border_width(GTK_CONTAINER (win), 5);

	vbox = gtk_vbox_new(FALSE, 5);
	gtk_container_add(GTK_CONTAINER (win), vbox);

	lbl = gtk_label_new(view->currPath);
	view->dirlabel = lbl;
	gtk_box_pack_start(GTK_BOX (vbox), lbl, FALSE, FALSE, 0);
	gtk_widget_show(lbl);

	scr = gtk_scrolled_window_new(NULL, NULL);
	view->mscr = scr;
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (scr), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_widget_set_usize(GTK_WIDGET (scr), 380, 360);
	gtk_box_pack_start(GTK_BOX (vbox), scr, FALSE, FALSE, 0);

	clist = gtk_clist_new_with_titles(4, titles);
	view->clist = clist;
	gtk_clist_set_column_width(GTK_CLIST (clist), 0, 120);
	gtk_clist_set_column_width(GTK_CLIST (clist), 1, 120);
	gtk_clist_set_column_width(GTK_CLIST (clist), 2, 60);
	gtk_clist_set_column_width(GTK_CLIST (clist), 3, 10);
	gtk_clist_set_column_justification(GTK_CLIST (clist), 0, GTK_JUSTIFY_LEFT);
	gtk_clist_set_column_justification(GTK_CLIST (clist), 1, GTK_JUSTIFY_LEFT);
	gtk_clist_set_column_justification(GTK_CLIST (clist), 2, GTK_JUSTIFY_RIGHT);
	gtk_clist_column_title_passive(GTK_CLIST (clist), 2);
	gtk_clist_set_column_justification(GTK_CLIST (clist), 3, GTK_JUSTIFY_LEFT);
	gtk_clist_set_auto_sort(GTK_CLIST (clist), FALSE);
	gtk_clist_set_selection_mode(GTK_CLIST (clist), GTK_SELECTION_EXTENDED);
	gtk_container_add(GTK_CONTAINER (scr), clist);
	view->sigBMouse = gtk_signal_connect_after(GTK_OBJECT (clist), "button_press_event", (GtkSignalFunc) mouseClick, (gpointer) view);
	view->sigBKey = gtk_signal_connect(GTK_OBJECT (clist), "key_press_event", (GtkSignalFunc) keyClick, (gpointer) view);
	gtk_signal_connect(GTK_OBJECT (clist), "select_row", (GtkSignalFunc) doImagePreview, (gpointer) view);
	gtk_widget_show(clist);
	gtk_widget_show(scr);
	GTK_WINDOW (win)->focus_widget = clist;
	hbox = gtk_fixed_new();
	gtk_widget_set_usize(GTK_WIDGET (hbox), 390, 80);
	gtk_box_pack_start(GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
	
	fra = gtk_frame_new(_("Controls"));
	gtk_widget_set_usize(GTK_WIDGET (fra), 210, 80);
	gtk_fixed_put(GTK_FIXED (hbox), fra, 0, 0);
	gtk_widget_show(fra);

	fra = gtk_frame_new(_("Preview"));
	gtk_widget_set_usize(GTK_WIDGET (fra), 170, 80);
	gtk_fixed_put(GTK_FIXED (hbox), fra, 220, 0);
	gtk_widget_show(fra);
		
	btn = gtk_button_new_with_label(_("Open"));
	gtk_tooltips_set_tip(view->tips, btn, _("Open the file"), NULL);
	gtk_signal_connect(GTK_OBJECT (btn), "clicked", (GtkSignalFunc) viewPic, (gpointer) view);
	gtk_widget_set_usize(GTK_WIDGET (btn), 90, 25);
	gtk_fixed_put(GTK_FIXED (hbox), btn, 10, 15);
	gtk_widget_show(btn);

	btn = gtk_button_new_with_label(_("SlideShow"));
	gtk_tooltips_set_tip(view->tips, btn, _("Start the slideshow"), NULL);
	view->slideBtn = btn;
	view->sigSlide = gtk_signal_connect(GTK_OBJECT (btn), "clicked", (GtkSignalFunc) startSlide, (gpointer) view);
	gtk_widget_set_usize(GTK_WIDGET (btn), 90, 25);
	gtk_fixed_put(GTK_FIXED (hbox), btn, 110, 15);
	gtk_widget_show(btn);

	btn = gtk_button_new_with_label(_("Configure"));
	gtk_tooltips_set_tip(view->tips, btn, _("Configure gView"), NULL);
	gtk_signal_connect(GTK_OBJECT (btn), "clicked", (GtkSignalFunc) configWin, (gpointer) view);
	gtk_widget_set_usize(GTK_WIDGET (btn), 90, 25);
	gtk_fixed_put(GTK_FIXED (hbox), btn, 10, 45);
	gtk_widget_show(btn);

	btn = gtk_button_new_with_label(_("Quit"));
	gtk_tooltips_set_tip(view->tips, btn, _("Quit gView"), NULL);
	gtk_signal_connect(GTK_OBJECT (btn), "clicked", (GtkSignalFunc) quit, (gpointer) view);
	gtk_widget_set_usize(GTK_WIDGET (btn), 90, 25);
	gtk_fixed_put(GTK_FIXED (hbox), btn, 110, 45);
	gtk_widget_show(btn);
	
	pix = gtk_preview_new(GTK_PREVIEW_COLOR);
	view->preview = pix;
	gtk_widget_set_usize(GTK_WIDGET (pix), 160, 60);
	gtk_fixed_put(GTK_FIXED (hbox), pix, 225, 15);
	gtk_widget_show(pix);

	gtk_widget_show(hbox);

	gtk_widget_show(vbox);
	gtk_widget_show(win);

}
