/* Copyright (C) 1999 Southern Gold Development <tangomanrulz@geocities.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "intl.h"

#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <gdk/gdkkeysyms.h>
#include <gdk_imlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <signal.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/wait.h>
#ifdef HAVE_SGC
#include <sgc/sgc.h>
#else
#define MAX_BUFF 1024
#endif /* HAVE_SGC */

#ifdef IS_PIC_USER
#include "back.xpm"
#endif /* IS_PIC_USER */

typedef struct _sgview sgview;
typedef struct _viewer viewer;
typedef struct _config config;
typedef struct _back back;
typedef struct _ren ren;
typedef struct _msgbuffer nmsg;

struct _sgview {
/* Browser window widgets */
	GtkWidget *main;
	GtkWidget *mscr;
	GtkWidget *clist;
	GtkWidget *dirlabel;
	GtkWidget *slideBtn;
	GtkWidget *preview;
	gint sigBKey;
	gint sigBMouse;
/* Viewer window widgets */
	GtkWidget *setrsp;
	GtkWidget *disp;
	GtkWidget *scr;
	GtkWidget *draw;
	GtkObject *adj;
	GdkImlibImage *currPic;
	GdkImlibImage *backPic;
	gint sigSlide;
	gint sigConf;
	gint sigKey;
	gint currX;
	gint currY;
/* General stuff */
	gchar *currPath;
	GList *slide;
	GList *currSlide;
	guint timeout;
	gint slideTime;
	guchar statCh;
	gfloat defGamma;
	GtkTooltips *tips;
	gchar *defBackColor;
#ifdef HAVE_WORKING_MSG
	gint dqueue;
	guint msgTimer;
#endif /* HAVE_WORKING_MSG */
};

struct _config {
	sgview *view;

	GtkWidget *rsp;
	GtkWidget *timeOut;
	GtkWidget *hiddenFiles;
	GtkWidget *defaultGamma;
	GtkWidget *backColor;
	GtkWidget *colDiag;
	GSList *defaultRender;
};

struct _ren {
	sgview *view;
	
	GtkWidget *rsp;
	GtkWidget *ent;
	GtkWidget *browser;
};

typedef enum {
	GVIEW_UP_STEP,
	GVIEW_DOWN_STEP,
	GVIEW_UP_PAGE,
	GVIEW_DOWN_PAGE
} GViewMoveType;

#ifdef HAVE_WORKING_MSG
struct _msgbuffer {
	long mtype;
	gchar mtext[MAX_BUFF];
};
#endif /* HAVE_WORKING_MSG */

typedef void (*sgImageSetFunc) (GdkImlibImage *im, GdkImlibColorModifier *mod);

/* Cross file function prototypes */

/* General operations and tools */
gchar *getLine(FILE *fHnd);
void sortRel(gchar **work);
void handleFile(sgview *view, gchar *file);
gint ipcIncoming(gpointer data);
void killDiag(GtkWidget *widget, gpointer data);
void gloaddir(sgview *view);
void loadPic(sgview *view, gchar *file);
void popupPic(sgview *view, GdkEventButton *event);
void createWin(sgview *view);

/* Viewer window */
void configWin(GtkWidget *widget, gpointer data);
void setBack(GtkWidget *widget, gpointer data);
void setDialog(GtkWidget *widget, gpointer data);
void redDialog(GtkWidget *widget, gpointer data);
void blueDialog(GtkWidget *widget, gpointer data);
void greenDialog(GtkWidget *widget, gpointer data);
void gammaSet(GtkWidget *widget, gpointer data);
void saveImageDialog(GtkWidget *widget, gpointer data);
gint doDragPic(GtkWidget *widget, GdkEventMotion *event, gpointer data);
gint startDragPic(GtkWidget *widget, GdkEventButton *event, gpointer data);
gint stopDragPic(GtkWidget *widget, GdkEventButton *event, gpointer data);

/* Browser window */
void enqueueSlide(GtkWidget *widget, gpointer data);
void startSlide(GtkWidget *widget, gpointer data);
void stopSlide(GtkWidget *widget, gpointer data);
void pauseSlide(GtkWidget *widget, gpointer data);
void resumeSlide(GtkWidget *widget, gpointer data);
void clearSlide(GtkWidget *widget, gpointer data);
void viewSlide(GtkWidget *widget, gpointer data);

void menuOpenURL(GtkWidget *widget, gpointer data);
void menuDelete(GtkWidget *widget, gpointer data);
void menuRename(GtkWidget *widget, gpointer data);
