/* Copyright (C) 1999 Southern Gold Development <tangomanrulz@geocities.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "gview.h"

gint resizePic(GtkWidget *widget, GdkEvent *event, gpointer data) {
	sgview *view;
	gchar *title;
	gint rx, ry;

	view = (sgview *) data;
	rx = widget->allocation.width;
	ry = widget->allocation.height;
	title = g_strdup_printf("gView - %s - %dx%d", view->currPic->filename, rx, ry);
	gtk_window_set_title(GTK_WINDOW (view->disp), title);
	g_free(title);
	gdk_imlib_apply_image(view->currPic, widget->window);
	return FALSE;
}

void killDiag(GtkWidget *widget, gpointer data) {
	sgview *view;

	view = (sgview *) data;
	if (view->setrsp)
		view->setrsp = NULL;
}

void popupPic(sgview *view, GdkEventButton *event) {
	GtkWidget *menu, *submenu, *item;

	menu = gtk_menu_new();
	submenu = gtk_menu_new();

	item = gtk_menu_item_new_with_label(_("Image Levels"));
	gtk_signal_connect(GTK_OBJECT (item), "activate", (GtkSignalFunc) setDialog, (gpointer) view);
	gtk_menu_append(GTK_MENU (submenu), item);
	gtk_widget_show(item);

	item = gtk_menu_item_new_with_label(_("Red Levels"));
	gtk_signal_connect(GTK_OBJECT (item), "activate", (GtkSignalFunc) redDialog, (gpointer) view);
	gtk_menu_append(GTK_MENU (submenu), item);
	gtk_widget_show(item);

	item = gtk_menu_item_new_with_label(_("Blue Levels"));
	gtk_signal_connect(GTK_OBJECT (item), "activate", (GtkSignalFunc) blueDialog, (gpointer) view);
	gtk_menu_append(GTK_MENU (submenu), item);
	gtk_widget_show(item);

	item = gtk_menu_item_new_with_label(_("Green Levels"));
	gtk_signal_connect(GTK_OBJECT (item), "activate", (GtkSignalFunc) greenDialog, (gpointer) view);
	gtk_menu_append(GTK_MENU (submenu), item);
	gtk_widget_show(item);

	item = gtk_menu_item_new_with_label(_("Manipulate Image"));
	gtk_menu_append(GTK_MENU (menu), item);
	gtk_menu_item_set_submenu(GTK_MENU_ITEM (item), submenu);
	gtk_widget_show(item);

    submenu = gtk_menu_new();

	item = gtk_menu_item_new_with_label(_("Start"));
	gtk_signal_connect(GTK_OBJECT (item), "activate", (GtkSignalFunc) startSlide, (gpointer) view);
	gtk_menu_append(GTK_MENU (submenu), item);
	gtk_widget_show(item);

	item = gtk_menu_item_new_with_label(_("Stop"));
	gtk_signal_connect(GTK_OBJECT (item), "activate", (GtkSignalFunc) stopSlide, (gpointer) view);
	gtk_menu_append(GTK_MENU (submenu), item);
	gtk_widget_show(item);

	item = gtk_menu_item_new_with_label(_("Pause"));
	gtk_signal_connect(GTK_OBJECT (item), "activate", (GtkSignalFunc) pauseSlide, (gpointer) view);
	gtk_menu_append(GTK_MENU (submenu), item);
	gtk_widget_show(item);

	item = gtk_menu_item_new_with_label(_("Resume"));
	gtk_signal_connect(GTK_OBJECT (item), "activate", (GtkSignalFunc) resumeSlide, (gpointer) view);
	gtk_menu_append(GTK_MENU (submenu), item);
	gtk_widget_show(item);

	item = gtk_menu_item_new_with_label(_("SlideShow"));
	gtk_menu_append(GTK_MENU (menu), item);
	gtk_menu_item_set_submenu(GTK_MENU_ITEM (item), submenu);
	gtk_widget_show(item);

	item = gtk_menu_item_new_with_label(_("Set As Background"));
	gtk_signal_connect(GTK_OBJECT (item), "activate", (GtkSignalFunc) setBack, (gpointer) view);
	gtk_menu_append(GTK_MENU (menu), item);
	gtk_widget_show(item);

	item = gtk_menu_item_new_with_label(_("Save Image As..."));
	gtk_signal_connect(GTK_OBJECT (item), "activate", (GtkSignalFunc) saveImageDialog, (gpointer) view);
	gtk_menu_append(GTK_MENU (menu), item);
	gtk_widget_show(item);

	item = gtk_menu_item_new_with_label(_("Close Image"));
	gtk_signal_connect_object(GTK_OBJECT (item), "activate", (GtkSignalFunc) gtk_widget_destroy, (GtkObject *) view->disp);
	gtk_menu_append(GTK_MENU (menu), item);
	gtk_widget_show(item);

	gtk_menu_popup(GTK_MENU (menu), NULL, NULL, NULL, NULL, event->button, event->time);
}

void navImage(sgview *view, gboolean forward) {
	gint row, mod;
	gint maxP, rowH, rowP, winT, winB;
	gfloat newP;
	GtkAdjustment *adj;
	gchar *file;

	row = GTK_CLIST (view->clist)->focus_row;
	if (forward) {
		mod = 1;
		if ((row == -1) || (row == (GTK_CLIST (view->clist)->rows) - 1))
			return;
	}
	else {
		mod = -1;
		if ((row == -1) || (row == 0))
			return;
	}
	GTK_CLIST (view->clist)->focus_row += mod;
	gtk_clist_unselect_row(GTK_CLIST (view->clist), row, 0);
	row += mod;
	gtk_clist_select_row(GTK_CLIST (view->clist), row, 0);
	gtk_clist_get_text(GTK_CLIST (view->clist), row, 0, &file);
	if ((file[strlen(file) - 1] != '/') && (file[strlen(file) - 1] != ')'))
		handleFile(view, file);
	adj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW (view->mscr));
	maxP = adj->upper;
	winT = adj->value;
	winB = winT + view->mscr->allocation.height;
	rowH = maxP / GTK_CLIST (view->clist)->rows;
	if (forward) {
		rowP = rowH * (GTK_CLIST (view->clist)->focus_row + 3);
		if (rowP > winB) {
			newP = winT + rowH;
			if (newP > adj->upper - adj->page_size)
				newP = adj->upper - adj->page_size;
			gtk_adjustment_set_value(GTK_ADJUSTMENT (adj), newP);
		}
	}
	else {
		rowP = rowH * (GTK_CLIST (view->clist)->focus_row);
		if (rowP < winT) {
			newP = winT - rowH;
			if (newP < adj->lower)
				newP = adj->lower;
			gtk_adjustment_set_value(GTK_ADJUSTMENT (adj), newP);
		}
	}	
}

void zoomImage(sgview *view, gboolean in) {
	gint xim, yim, xr, yr, hr, wr, dr, dx, dy;
	gboolean overs;

	xim = GTK_WIDGET (view->draw)->allocation.width;
	yim = GTK_WIDGET (view->draw)->allocation.height;
	gdk_window_get_geometry(GDK_ROOT_PARENT(), &xr, &yr, &wr, &hr, &dr);

	gtk_signal_handler_block(GTK_OBJECT (view->draw), view->sigConf);
	if (in) {
		xim *= 110;
		xim /= 100;
		yim *= 110;
		yim /= 100;
	}
	else {
		xim *= 90;
		xim /= 100;
		yim *= 90;
		yim /= 100;
	}
	dx = xim;
	dy = yim;
	overs = FALSE;
	if (xim + 85 > wr) {
		xim = wr - 85;
		overs = TRUE;
	}
	if (yim + 85 > hr) {
		yim = hr - 85;
		overs = TRUE;
	}
	gtk_widget_set_usize(GTK_WIDGET (view->draw), dx, dy);
	if (overs) {
		gtk_widget_set_usize(GTK_WIDGET (view->disp), xim + 23, yim + 23);
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (view->scr), GTK_POLICY_ALWAYS, GTK_POLICY_ALWAYS);
	}
	else {
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (view->scr), GTK_POLICY_NEVER, GTK_POLICY_NEVER);
		gtk_widget_set_usize(GTK_WIDGET (view->disp), xim + 4, yim + 4);
	}
	gtk_widget_set_usize(GTK_WIDGET (view->scr), dx, dy);
	while (gtk_events_pending())
		gtk_main_iteration();
	gdk_imlib_apply_image(view->currPic, view->draw->window);
	while (gtk_events_pending())
		gtk_main_iteration();
	gtk_signal_handler_unblock(GTK_OBJECT (view->draw), view->sigConf);
}

void doMove(sgview *view, GViewMoveType move, gboolean vert) {
	GtkAdjustment *adj;
	gfloat val, step, edge;
	gboolean flag;

	if (vert)
		adj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW (view->scr));
	else
		adj = gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW (view->scr));
	switch (move) {
		case GVIEW_UP_STEP:
			step = adj->step_increment;
			edge = adj->upper - adj->page_size;
			flag = TRUE;
			break;
		case GVIEW_DOWN_STEP:
			step = -(adj->step_increment);
			edge = adj->lower;
			flag = FALSE;
			break;
		case GVIEW_UP_PAGE:
			step = adj->page_increment;
			edge = adj->upper - adj->page_size;
			flag = TRUE;
			break;
		case GVIEW_DOWN_PAGE:
			step = -(adj->page_increment);
			edge = adj->lower;
			flag = FALSE;
			break;
		default:
			return;
			break;
	}
	val = adj->value + step;
	if (flag) {
		if (val > edge)
			val = edge;
	}
	else
		if (val < edge)
			val = edge;
	gtk_adjustment_set_value(adj, val);
}

gint viewKeys(GtkWidget *widget, GdkEventKey *event, gpointer data) {
	sgview *view;

	view = (sgview *) data;

	switch (event->keyval) {
		case GDK_V:
		case GDK_v:
			gdk_imlib_flip_image_vertical(view->currPic);
			gdk_imlib_apply_image(view->currPic, view->draw->window);
			break;
		case GDK_H:
		case GDK_h:
			gdk_imlib_flip_image_horizontal(view->currPic);
			gdk_imlib_apply_image(view->currPic, view->draw->window);
			break;
		case GDK_Q:
		case GDK_q:
			gtk_widget_destroy(view->disp);
			break;
		case GDK_Left:
			doMove(view, GVIEW_DOWN_STEP, FALSE);
			break;
		case GDK_Right:
			doMove(view, GVIEW_UP_STEP, FALSE);
			break;
		case GDK_Up:
			doMove(view, GVIEW_DOWN_STEP, TRUE);
			break;
		case GDK_Down:
			doMove(view, GVIEW_UP_STEP, TRUE);
			break;
		case GDK_Page_Up:
			if (event->state & GDK_SHIFT_MASK)
				doMove(view, GVIEW_DOWN_PAGE, FALSE);
			else
				doMove(view, GVIEW_DOWN_PAGE, TRUE);
			break;
		case GDK_Page_Down:
			if (event->state & GDK_SHIFT_MASK)
				doMove(view, GVIEW_UP_PAGE, FALSE);
			else
				doMove(view, GVIEW_UP_PAGE, TRUE);
			break;
		case GDK_n:
		case GDK_N:
			navImage(view, TRUE);
			break;
		case GDK_p:
		case GDK_P:
			navImage(view, FALSE);
			break;
		case GDK_plus:
		case GDK_equal:
		case GDK_KP_Add:
			zoomImage(view, TRUE);
			break;
		case GDK_minus:
		case GDK_KP_Subtract:
			zoomImage(view, FALSE);
			break;
		default:
			break;
	}
	gtk_signal_emit_stop_by_name(GTK_OBJECT (widget), "key_press_event");
	return TRUE;
}

void killView(GtkWidget *widget, gpointer data) {
	sgview *view;

	view = (sgview *) data;
	view->disp = NULL;
	if (view->setrsp)
		gtk_widget_destroy(view->setrsp);
	view->setrsp = NULL;
	if (view->backPic != view->currPic)
		gdk_imlib_kill_image(view->backPic);
	gdk_imlib_kill_image(view->currPic);
	view->currPic = NULL;
}

void loadPic(sgview *view, gchar *file) {
	GtkWidget *win, *scr, *draw;
	GdkImlibImage *im;
	GtkAdjustment *sadj;
	GdkCursor *curs;
	gchar *title, *dim, *fend;
	GdkImlibColorModifier mod;
	gint xim, yim, xr, yr, wr, hr, dr, place, depth,overs;

	gdk_window_get_geometry(GDK_ROOT_PARENT(), &xr, &yr, &wr, &hr, &dr);
	depth = gdk_visual_get_best_depth();

	im = gdk_imlib_load_image(file);
	if (!(im))
		return;
	xim = im->rgb_width;
	yim = im->rgb_height;
	
	overs = FALSE;
	if (xim + 85 > wr) {
		xim = wr - 85;
		overs = TRUE;
	}
	if (yim + 85 > hr) {
		yim = hr - 85;
		overs = TRUE;
	}
	
	if (view->currPic)
		if (view->backPic != view->currPic)
			gdk_imlib_kill_image(view->backPic);
		gdk_imlib_kill_image(view->currPic);
	view->currPic = im;
	view->backPic = view->currPic;
	
	if (view->main) {
		dim = g_strdup_printf("%dx%d", im->rgb_width, im->rgb_height); 
		if (view->statCh & 0x02)
			gtk_clist_set_text(GTK_CLIST (view->clist), GTK_CLIST(view->clist)->focus_row, 2, dim);
	}
	else
		dim = NULL;

	fend = rindex (file, '/');
	fend++;
	title = g_strconcat("gView - ", fend, " - ", dim, NULL);
	g_free (dim);

	if (view->disp) {
		if (view->setrsp)
			gtk_widget_destroy(view->setrsp);
		view->setrsp = NULL;
		gtk_signal_handler_block(GTK_OBJECT (view->disp), view->sigKey);
		gtk_signal_handler_block(GTK_OBJECT (view->clist), view->sigBKey);
		gtk_signal_handler_block(GTK_OBJECT (view->clist), view->sigBMouse);
		gtk_signal_handler_block(GTK_OBJECT (view->draw), view->sigConf);
		gtk_window_set_title(GTK_WINDOW (view->disp), title);
		if (overs)
			gtk_widget_set_usize(GTK_WIDGET (view->disp), xim + 23, yim + 23);
		else
			gtk_widget_set_usize(GTK_WIDGET (view->disp), xim + 4, yim + 4);
		gtk_widget_set_usize(GTK_WIDGET (view->scr), xim, yim);
		if (overs) 
			gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (view->scr), GTK_POLICY_ALWAYS, GTK_POLICY_ALWAYS);
		else
			gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (view->scr), GTK_POLICY_NEVER, GTK_POLICY_NEVER);
		sadj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW (view->scr));
		gtk_adjustment_set_value(sadj, 0);
		sadj = gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW (view->scr));
		gtk_adjustment_set_value(sadj, 0);
		gtk_drawing_area_size(GTK_DRAWING_AREA (view->draw), im->rgb_width, im->rgb_height);
		gtk_widget_set_usize(GTK_WIDGET (view->draw), im->rgb_width, im->rgb_height);
		gtk_widget_reset_shapes(GTK_WIDGET (view->draw));
		while (gtk_events_pending())
			gtk_main_iteration();
/*		val = GTK_ADJUSTMENT (view->adj)->value;*/
		place = view->defGamma * 256;
		gdk_imlib_get_image_modifier(im, &mod);
		mod.gamma = place;
		gdk_imlib_set_image_modifier(im, &mod);
		gdk_imlib_apply_image(im, view->draw->window);
		while (gtk_events_pending())
			gtk_main_iteration();
		gtk_signal_handler_unblock(GTK_OBJECT (view->disp), view->sigKey);
		gtk_signal_handler_unblock(GTK_OBJECT (view->clist), view->sigBMouse);
		gtk_signal_handler_unblock(GTK_OBJECT (view->clist), view->sigBKey);
		gtk_signal_handler_unblock(GTK_OBJECT (view->draw), view->sigConf);
	}
	else {
		view->setrsp = NULL;
		gtk_signal_handler_block(GTK_OBJECT (view->clist), view->sigBKey);
		gtk_signal_handler_block(GTK_OBJECT (view->clist), view->sigBMouse);
		win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		view->disp = win;
		gtk_window_set_title(GTK_WINDOW (win), title);
		gtk_window_set_policy(GTK_WINDOW (win), TRUE, TRUE, TRUE);
		gtk_window_set_position(GTK_WINDOW (win), GTK_WIN_POS_MOUSE);
		gtk_signal_connect(GTK_OBJECT (win), "destroy", (GtkSignalFunc) killView, (gpointer) view);
		view->sigKey = gtk_signal_connect(GTK_OBJECT (win), "key_press_event", (GtkSignalFunc) viewKeys, (gpointer) view);
		gtk_container_border_width(GTK_CONTAINER (win), 0);
		if (overs)
			gtk_widget_set_usize(GTK_WIDGET (win), xim + 23, yim + 23);
		else
			gtk_widget_set_usize(GTK_WIDGET (win), xim + 4, yim + 4);
		
/*		vbox = gtk_vbox_new(FALSE, 5);
		gtk_container_add(GTK_CONTAINER (win), vbox);*/
			
		scr = gtk_scrolled_window_new(NULL, NULL);
		view->scr = scr;
		gtk_container_border_width(GTK_CONTAINER (scr), 0);
		gtk_widget_set_usize(GTK_WIDGET (scr), xim, yim);
		if (overs) 
			gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (scr), GTK_POLICY_ALWAYS, GTK_POLICY_ALWAYS);
		else
			gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (scr), GTK_POLICY_NEVER, GTK_POLICY_NEVER);

/*		gtk_box_pack_start(GTK_BOX (vbox), scr, TRUE, TRUE, 0);*/
		gtk_container_add(GTK_CONTAINER (win), scr);
	
		draw = gtk_drawing_area_new();
		gtk_drawing_area_size(GTK_DRAWING_AREA (draw), im->rgb_width, im->rgb_height);
		view->draw = draw;
		gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW (scr), draw);
		gtk_signal_connect(GTK_OBJECT (draw), "button_press_event", (GtkSignalFunc) startDragPic, (gpointer) view);
		gtk_signal_connect(GTK_OBJECT (draw), "motion_notify_event", (GtkSignalFunc) doDragPic, (gpointer) view);
		gtk_signal_connect(GTK_OBJECT (draw), "button_release_event", (GtkSignalFunc) stopDragPic, (gpointer) view);
		gtk_widget_set_events(draw, GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK);
		
		gtk_widget_show(draw);
		gtk_widget_show(scr);
/*		gtk_widget_show(vbox);*/
	
/*		hbox = gtk_fixed_new();
		gtk_box_pack_start(GTK_BOX (vbox), hbox, FALSE, TRUE, 5);

		lbl = gtk_label_new(_("Adjust Gamma"));
		gtk_fixed_put(GTK_FIXED (hbox), lbl, 10, 10);
		gtk_widget_show(lbl);
	
		adj = gtk_adjustment_new(view->defGamma, 0.0, 10.0, 0.1, 1.0, 1.0);
		view->adj = adj;
		gtk_signal_connect(GTK_OBJECT (adj), "value-changed", (GtkSignalFunc) gammaSet, (gpointer) view);
		hsb = gtk_hscale_new(GTK_ADJUSTMENT (adj));
		gtk_widget_set_usize (GTK_WIDGET (hsb), 250, 25);
		gtk_fixed_put(GTK_FIXED (hbox), hsb, 100, 5);
	
		gtk_widget_show(hsb);
		gtk_widget_show(hbox);

		gtk_widget_show(vbox);*/
		gtk_widget_show(win);
		place = view->defGamma * 256;
		gdk_imlib_get_image_modifier(im, &mod);
		mod.gamma = place;
		gdk_imlib_set_image_modifier(im, &mod);
		gdk_imlib_apply_image(im, view->draw->window);
		while (gtk_events_pending())
			gtk_main_iteration();
		view->sigConf = gtk_signal_connect(GTK_OBJECT (draw), "configure-event", (GtkSignalFunc) resizePic, (gpointer) view);
		gtk_signal_handler_unblock(GTK_OBJECT (view->clist), view->sigBMouse);
		gtk_signal_handler_unblock(GTK_OBJECT (view->clist), view->sigBKey);
	}
	g_free(title);
	while (gtk_events_pending())
		gtk_main_iteration();
	if (overs)
		curs = gdk_cursor_new(GDK_HAND2);
	else
		curs = gdk_cursor_new(GDK_LEFT_PTR);
	gdk_window_set_cursor(view->draw->window,curs);
	gdk_cursor_destroy(curs);
	gtk_drawing_area_size(GTK_DRAWING_AREA (view->draw), im->rgb_width, im->rgb_height);
	gdk_imlib_apply_image(im, view->draw->window);
	if (view->main)
		gtk_widget_grab_focus(GTK_WIDGET (view->clist));
}
